
//   **************************************************************************
//   *                                                                        *
//   *                      ! ! ! A T T E N T I O N ! ! !                     *
//   *                                                                        *
//   *  This file was created automatically by GaudiObjDesc, please do not    *
//   *  delete it or edit it by hand.                                         *
//   *                                                                        *
//   *  If you want to change this file, first change the corresponding       *
//   *  xml-file and rerun the tools from GaudiObjDesc (or run make if you    *
//   *  are using it from inside a Gaudi-package).                            *
//   *                                                                        *
//   **************************************************************************

#ifndef LHCbKernel_STChannelID_H
#define LHCbKernel_STChannelID_H 1

// Include files
#include "GaudiKernel/boost_allocator.h"
#include <ostream>

// Forward declarations

namespace LHCb 
{

  // Forward declarations

  /** @class STChannelID STChannelID.h
   *
   * Channel ID for class for ST 
   *
   * @author M Needham
   * created Tue Oct 24 11:35:03 2006
   * 
   */

  class STChannelID
  {
  public:

    /// types of sub-detector channel ID
    enum detType{ typeTT = 0,
                  typeIT = 1
      };
  
    /// constructor with station, layer, detRegion, sector , strip,
    STChannelID(const unsigned int iType,
                const unsigned int iStation,
                const unsigned int iLayer,
                const unsigned int iDetRegion,
                const unsigned int iSector,
                const unsigned int iStrip);
  
    /// copy constructor
    STChannelID(const STChannelID& aChannelID);
  
    /// constructor with int
    STChannelID(int id) : m_channelID(id) {}
  
    /// Default Constructor
    STChannelID() : m_channelID(0) {}
  
    /// Default Destructor
     ~STChannelID() {}
  
    /// Fill the ASCII output stream
   std::ostream& fillStream(std::ostream& s) const;
  
    /// cast
    operator int() const;
  
    /// get the type
    unsigned int type() const;
  
    /// test whether TT or not
    bool isTT() const;
  
    /// test whether IT or not
    bool isIT() const;
  
    /// Retrieve const  OT Channel ID
    unsigned int channelID() const;
  
    /// Update  OT Channel ID
    void setChannelID(unsigned int value);
  
    /// Retrieve strip
    unsigned int strip() const;
  
    /// Retrieve sector
    unsigned int sector() const;
  
    /// Retrieve detRegion
    unsigned int detRegion() const;
  
    /// Retrieve layer
    unsigned int layer() const;
  
    /// Retrieve station
    unsigned int station() const;
  
    /// Retrieve Unique layer
    unsigned int uniqueLayer() const;
  
    /// Retrieve Unique detRegion
    unsigned int uniqueDetRegion() const;
  
    /// Retrieve Unique layer
    unsigned int uniqueSector() const;
  
  
  #ifndef _WIN32
    /// operator new
    static void* operator new ( size_t size )
    {
      return ( sizeof(STChannelID) == size ? 
               boost::singleton_pool<STChannelID, sizeof(STChannelID)>::malloc() :
               ::operator new(size) );
    }
  
    /// placement operator new
    /// it is needed by libstdc++ 3.2.3 (e.g. in std::vector)
    /// it is not needed in libstdc++ >= 3.4
    static void* operator new ( size_t size, void* pObj )
    {
      return ::operator new (size,pObj);
    }
  
    /// operator delete
    static void operator delete ( void* p )
    {
      boost::singleton_pool<STChannelID, sizeof(STChannelID)>::is_from(p) ?
      boost::singleton_pool<STChannelID, sizeof(STChannelID)>::free(p) :
      ::operator delete(p);
    }
  
    /// placement operator delete
    /// not sure if really needed, but it does not harm
    static void operator delete ( void* p, void* pObj )
    {
      ::operator delete (p, pObj);
    }
  #endif
  protected:

  private:

    /// Offsets of bitfield channelID
    enum channelIDBits{stripBits           = 0,
                       sectorBits          = 10,
                       detRegionBits       = 15,
                       layerBits           = 18,
                       stationBits         = 21,
                       typeBits            = 23};
  
    /// Bitmasks for bitfield channelID
    enum channelIDMasks{stripMask           = 0x3FFL,
                        sectorMask          = 0x7C00L,
                        detRegionMask       = 0x38000L,
                        layerMask           = 0x1C0000L,
                        stationMask         = 0x600000L,
                        typeMask            = 0x800000L,
                        uniqueLayerMask     = layerMask + stationMask,
                        uniqueDetRegionMask = detRegionMask + layerMask + stationMask,
                        uniqueSectorMask    = sectorMask + detRegionMask + layerMask + stationMask
                       };
  
  
    unsigned int m_channelID; ///< OT Channel ID
  
  }; // class STChannelID

  inline std::ostream& operator<< (std::ostream& str, const STChannelID& obj)
  {
    return obj.fillStream(str);
  }
  
  inline std::ostream & operator << (std::ostream & s, LHCb::STChannelID::detType e) {
    switch (e) {
      case LHCb::STChannelID::typeTT : return s << "typeTT";
      case LHCb::STChannelID::typeIT : return s << "typeIT";
      default : return s << "ERROR wrong value for enum LHCb::STChannelID::detType";
    }
  }
  
  
} // namespace LHCb;

// -----------------------------------------------------------------------------
// end of class
// -----------------------------------------------------------------------------

// Including forward declarations

inline LHCb::STChannelID::STChannelID(const unsigned int iType,
                                      const unsigned int iStation,
                                      const unsigned int iLayer,
                                      const unsigned int iDetRegion,
                                      const unsigned int iSector,
                                      const unsigned int iStrip) 
{
    

          m_channelID = (iType << typeBits) +
                        (iStation << stationBits) + 
                        (iLayer << layerBits) + 
                        (iDetRegion << detRegionBits) + 
                        (iSector <<  sectorBits) + 
                        (iStrip << stripBits);
        
}

inline LHCb::STChannelID::STChannelID(const STChannelID& aChannelID) 
{

          m_channelID = aChannelID.m_channelID;
        
}

inline std::ostream& LHCb::STChannelID::fillStream(std::ostream& s) const
{
  s << "{ " << "channelID :	" << m_channelID << std::endl << " }";
  return s;
}


inline unsigned int LHCb::STChannelID::channelID() const 
{
  return m_channelID;
}

inline void LHCb::STChannelID::setChannelID(unsigned int value) 
{
  m_channelID = value;
}

inline unsigned int LHCb::STChannelID::strip() const
{
  return (unsigned int)((m_channelID & stripMask) >> stripBits);
}

inline unsigned int LHCb::STChannelID::sector() const
{
  return (unsigned int)((m_channelID & sectorMask) >> sectorBits);
}

inline unsigned int LHCb::STChannelID::detRegion() const
{
  return (unsigned int)((m_channelID & detRegionMask) >> detRegionBits);
}

inline unsigned int LHCb::STChannelID::layer() const
{
  return (unsigned int)((m_channelID & layerMask) >> layerBits);
}

inline unsigned int LHCb::STChannelID::station() const
{
  return (unsigned int)((m_channelID & stationMask) >> stationBits);
}

inline unsigned int LHCb::STChannelID::uniqueLayer() const
{
  return (unsigned int)((m_channelID & uniqueLayerMask) >> layerBits);
}

inline unsigned int LHCb::STChannelID::uniqueDetRegion() const
{
  return (unsigned int)((m_channelID & uniqueDetRegionMask) >> detRegionBits);
}

inline unsigned int LHCb::STChannelID::uniqueSector() const
{
  return (unsigned int)((m_channelID & uniqueSectorMask) >> sectorBits);
}

inline LHCb::STChannelID::operator int() const 
{
return m_channelID;
}

inline unsigned int LHCb::STChannelID::type() const 
{
return (m_channelID & typeMask) >> typeBits;
}

inline bool LHCb::STChannelID::isTT() const 
{
return (type() == LHCb::STChannelID::typeTT ? true : false);
}

inline bool LHCb::STChannelID::isIT() const 
{
return (type() == LHCb::STChannelID::typeIT ? true : false);
}


#endif ///LHCbKernel_STChannelID_H
