
#include <boost/lexical_cast.hpp>
#include <boost/assign/std/vector.hpp>
#include "genx.h"
#include <string>
#include <iostream>

/*
  g++ -I/afs/cern.ch/sw/lcg/external/Boost/1.32.0/slc3_ia32_gcc323/include/boost-1_32 -I/afs/cern.ch/sw/Gaudi/releases/GAUDI/GAUDI_v18r9/GaudiKernel/v20r10/ -Igenx/ itCond.cpp genx/libgenx.a -o itCond
*/

using namespace boost::assign;
using namespace boost;

#include "STChannelID.h"

using namespace LHCb;
const unsigned int nHybrid = 8;


void oops(genxWriter w)
{
  fprintf(stderr, "oops ERROR %s\n", genxLastErrorMessage(w));
  exit(1);
}

extern void vectorToString( std::string& output , std::vector<STChannelID>& channels);
 
int main(){


  std::string hybridsPerBoard = boost::lexical_cast<std::string>(nHybrid);

  std::vector<int> sectorChannels;
  sectorChannels += 1 ,1 ,1 ,1 ,1 ,1 ,1, 1,
                     1 ,1 ,1 ,1 ,1 ,1 ,1, 1,
                     1 ,1 ,1 ,1 ,1 ,1 ,1, 1,
                     1 ,1 ,1 ,1 ,4 ,4 ,4, 4,
                     4 ,4 ,4 ,4 ,4 ,4 ,4, 4,
                     4 ,4 ,4 ,4 ,4 ,4 ,4, 4,
                     4 ,4 ,4 ,4 ,4 ,4 ,4, 4,
                     2 ,2 ,2 ,2 ,2 ,2 ,2, 2,
                     2 ,2 ,2 ,2 ,2 ,2 ,2, 2,
                     2 ,2 ,2 ,2 ,2 ,2 ,2, 2,
                     2 ,2 ,2 ,2, 3, 3, 3, 3,
                     3 ,3 ,3 ,3 ,3 ,3 ,3, 3,
                     3 ,3 ,3 ,3 ,3 ,3 ,3, 3,
                     3, 3 ,3 ,3 ,3 ,3 ,3 ,3 ;


  std::vector<int> layerChannels;  
  layerChannels +=  1,1,1,1,2,2,2,2,
                    3,3,3,3,4,4,4,4,
                    2,2,3,3,3,4,4,4,
                    1,1,1,2,1,1,1,2,
                    1,1,1,1,2,2,2,2,
                    3,3,3,3,4,4,4,4,
                    2,2,3,3,3,4,4,4,
                    1,1,1,1,2,2,2,2,
                    3,3,3,3,4,4,4,4,
                    2,2,3,3,3,4,4,4,
                    1,1,1,2,1,1,1,2,
                    1,1,1,1,2,2,2,2,
                    3,3,3,3,4,4,4,4,
                    2,2,3,3,3,4,4,4;

  std::vector<int> waferChannels;  
  waferChannels  += 4,5,6,7,4,5,6,7,
                    4,5,6,7,4,5,6,7,
                    1,2,1,2,3,1,2,3,
                    1,2,3,3,5,6,7,5,

                    1,2,3,4,1,2,3,4,
                    1,2,3,4,1,2,3,4,
                    6,7,5,6,7,5,6,7,

                    1,2,3,4,1,2,3,4,
                    1,2,3,4,1,2,3,4,
                    6,7,5,6,7,5,6,7,
                    5,6,7,5,1,2,3,3,

                    4,5,6,7,4,5,6,7,
                    4,5,6,7,4,5,6,7,
                    1,2,1,2,3,1,2,3;

                                                                               
  // make vector of for station 3 channelIDs...
  std::vector<STChannelID> channels;
  for (unsigned int i = 0; i < waferChannels.size() ; ++i){
    STChannelID aChan(1,1,layerChannels[i], sectorChannels[i], waferChannels[i], 0 );
    channels.push_back(aChan);
  }
  // make string
  std::string station3Map;
  vectorToString(station3Map, channels);

  // station 4
  channels.clear();
  for (unsigned int i = 0; i < waferChannels.size() ; ++i){
    STChannelID aChan(1,2,layerChannels[i], sectorChannels[i], waferChannels[i], 0 );
    channels.push_back(aChan);
  }
  // make string
  std::string station4Map;
  vectorToString(station4Map, channels);

  // station 5
  channels.clear();
  for (unsigned int i = 0; i < waferChannels.size() ; ++i){
    STChannelID aChan(1,3,layerChannels[i], sectorChannels[i], waferChannels[i], 0 );
    channels.push_back(aChan);
  }

  std::string station5Map;
  vectorToString(station5Map, channels);

  std::vector<std::string> stations;
  stations += "IT1Map", "IT2Map", "IT3Map";

  std::string stationList;
  for (std::vector<std::string>::iterator iterS = stations.begin(); 
       iterS != stations.end(); ++iterS){
    stationList += " "+(*iterS);
  } // iterS


  std::vector<std::string> output; 
  output += station3Map, station4Map , station5Map;

  typedef const unsigned char* gTYPE; 

  genxWriter w = genxNew(NULL, NULL, NULL);

  genxStatus status;
  
  genxElement param;
  genxElement paramVector;
  genxElement dddb;
  genxElement condition;
  genxElement catalog;
 
  genxAttribute name;
  genxAttribute type; 
  genxAttribute comment;
  genxAttribute classID;

  if (!( paramVector = genxDeclareElement(w, NULL, (gTYPE)"paramVector", &status))) oops(w);
  if (!(param = genxDeclareElement(w, NULL,(gTYPE) "param", &status))) oops(w);

  if (!(dddb =genxDeclareElement(w, NULL, (gTYPE)"DDDB", &status))) oops(w);
  if (!( catalog = genxDeclareElement(w, NULL, (gTYPE)"catalog", &status))) oops(w);
  if (!( condition = genxDeclareElement(w, NULL, (gTYPE)"condition", &status))) oops(w);

  if (!(name = genxDeclareAttribute(w, NULL,(gTYPE) "name", &status))) oops(w);
  if (!(comment = genxDeclareAttribute(w, NULL,(gTYPE) "comment", &status))) oops(w);
  if (!(classID = genxDeclareAttribute(w, NULL,(gTYPE) "classID", &status))) oops(w);
  if (!(type = genxDeclareAttribute(w, NULL,(gTYPE) "type", &status))) oops(w);

  std::string detector = "IT";

  genxStartDocFile(w, stdout);
  genxStartElement(dddb);
    genxAddText(w, (gTYPE)"\n");
    genxStartElement(catalog);
      genxAddAttribute(name, (gTYPE)detector.c_str());
      genxAddText(w, (gTYPE)"\n");
      genxStartElement(condition);
      genxAddAttribute(name,(gTYPE)"ReadoutMap");
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(paramVector);
        genxAddAttribute(name,(gTYPE)"stations");
        genxAddAttribute(type,(gTYPE)"string");
        genxAddAttribute(comment,(gTYPE)"List of stations");
	genxAddText(w,(gTYPE)stationList.c_str());
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(param);
        genxAddAttribute(name,(gTYPE)"hybridsPerBoard");
        genxAddAttribute(type,(gTYPE)"int");  
        genxAddAttribute(comment,(gTYPE)"# hybrids per Tell1 board");
	genxAddText(w,(gTYPE)hybridsPerBoard.c_str());
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(param);
        genxAddAttribute(name,(gTYPE)"nBoard");
        genxAddAttribute(type,(gTYPE)"int");  
        genxAddAttribute(comment,(gTYPE)"# Tell1 board in a IT station");
	genxAddText(w,(gTYPE)"14");
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");

      for (unsigned int i = 0 ; i < stations.size(); ++i){
        genxStartElement(paramVector);
          genxAddAttribute(name,(gTYPE)stations[i].c_str());
          genxAddAttribute(type,(gTYPE)"int");  
	  std::string tmp = "Readout Mapping for " + stations[i];
          genxAddAttribute(comment,(gTYPE)tmp.c_str());
          genxAddText(w,(gTYPE)output[i].c_str());
          genxAddText(w, (gTYPE)"\n");
        genxEndElement(w);
	genxAddText(w, (gTYPE)"\n");
      } //
 
      genxAddText(w, (gTYPE)"\n");
    genxEndElement(w);
    genxAddText(w, (gTYPE)"\n");
   genxEndElement(w);
  genxAddText(w, (gTYPE)"\n");
  genxEndElement(w);
  genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  // output for tex
  std::cout << std::endl;
  std::ostringstream mess;
  int vecLoc = 0;
  for (unsigned int it = 0; it <14; ++it  ){
    std::ostringstream mess;
    mess << it;
    for (unsigned int ih = 0 ; ih < 8 ; ++ih, ++vecLoc){
      mess << " & (" << layerChannels[vecLoc] << "," 
                     << sectorChannels[vecLoc] << "," 
                     << waferChannels[vecLoc]<< ") ";   
    }
    mess << "\\ \hline" ;
    std::cout << mess.str() << std::endl;
  }
   
  return 0;
}


extern void vectorToString( std::string& output , std::vector<STChannelID>& channels){

  for (unsigned int i = 0; i < channels.size() ; ++i){
    output += " " + boost::lexical_cast<std::string>(channels[i].channelID());        
    //if ((i+1) % nHybrid == 0) output += " \n";  
  }
}
