#include <boost/lexical_cast.hpp>
#include <boost/assign/std/vector.hpp>
#include "genx.h"
#include <string>
#include <iostream>

/*
  g++ -I/afs/cern.ch/sw/lcg/external/Boost/1.32.0/slc3_ia32_gcc323/include/
  boost-1_32 -I../genx/ ttAlignment.cpp ../genx/libgenx.a -o ttAlignment
*/

using namespace boost::assign;
using namespace boost;

typedef const unsigned char* gTYPE;

genxWriter w = genxNew(NULL, NULL, NULL);
genxStatus status;  


void oops(genxWriter w)
{
  fprintf(stderr, "oops ERROR %s\n", genxLastErrorMessage(w));
  exit(1);
}

extern void makeAlignmentCondition( const std::string& detectorName );

int main(){

  std::string detector = "TT";
  std::vector<std::string> station;
  station += "TTa", "TTb";
  std::vector<std::string> layer;
  layer += "X", "U", "V", "X";
  std::vector<std::string> region;
  region += "R1", "R2", "R3";
  std::vector<std::string> module;
  module += "Module1", "Module2", "Module3", "Module4",
            "Module5", "Module6", "Module7";
  std::vector<std::string> sensor;
  sensor += "Sensor1", "Sensor2", "Sensor3", "Sensor4",
            "Sensor5", "Sensor6", "Sensor7";
  std::vector<std::string> tb;
  tb += "T", "B";

  genxElement dddb;
  genxElement catalog;
  genxElement conditionref;
  genxAttribute name;
  genxAttribute href;

  if (!(dddb =genxDeclareElement(w, NULL, (gTYPE)"DDDB", &status))) oops(w);
  if (!( catalog = genxDeclareElement(w, NULL, (gTYPE)"catalog", &status)))
    oops(w);
  if (!( conditionref = genxDeclareElement(w, NULL, (gTYPE)"conditionref", 
                                           &status))) oops(w);
  if (!(name = genxDeclareAttribute(w, NULL,(gTYPE) "name", &status))) oops(w);
  if (!(href = genxDeclareAttribute(w, NULL,(gTYPE) "href", &status))) oops(w);

  FILE *f = fopen("Detectors.xml", "w");
  genxStartDocFile(w, f);
  genxStartElement(dddb);
  genxAddText(w, (gTYPE)"\n");

  // Condition for whole detector
  makeAlignmentCondition( detector + "System"); 

  // Loop over stations
  for ( int i = 0; i != 2; ++i ) {
    makeAlignmentCondition( station[i] );
  }
      
  // Loop over stations
  for ( int i = 0; i != 2; ++i ) {
    // Loop over layers
    for ( int j = 0; j != 2; ++j ) {
      makeAlignmentCondition( station[i]+layer[2*i+j]+"Layer" );
    }
  }
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  f = fopen("Modules.xml", "w");
  genxStartDocFile(w, f);
  genxStartElement(dddb);
  genxAddText(w, (gTYPE)"\n");      
  // Loop over stations
  for ( int i = 0; i != 2; ++i ) {
    // Loop over layers
    for ( int j = 0; j != 2; ++j ) {
      // Loop over regions
      for ( int k = 0; k != 3; ++k ) {
        int nModules = (region[k]=="R2") ? 3 + (2*i) : 6 ;
        // Loop over modules
        for ( int l = 0; l != nModules; ++l ) {
          // Loop over top/bottom
          for ( int m = 0; m != 2; ++m ) {
            makeAlignmentCondition( station[i]+layer[2*i+j]+"Layer"
                                    +region[k]+module[l]+tb[m] );
          }
        }
      }
    }
  }
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  f = fopen("Sensors.xml", "w");
  genxStartDocFile(w, f);
  genxStartElement(dddb);
  genxAddText(w, (gTYPE)"\n");      
  // Loop over stations
  for ( int i = 0; i != 2; ++i ) {
    // Loop over layers
    for ( int j = 0; j != 2; ++j ) {
      // Loop over regions
      for ( int k = 0; k != 3; ++k ) {
        int nModules = (region[k]=="R2") ? 3 + (2*i) : 6 ;
        // Loop over modules
        for ( int l = 0; l != nModules; ++l ) {
          // Loop over top/bottom
          for ( int m = 0; m != 2; ++m ) {
            // loop over sensors
            for (int n = 0; n != 7; ++n ) {                 
              makeAlignmentCondition( station[i]+layer[2*i+j]+"Layer"
                                      +region[k]+module[l]+tb[m]+sensor[n]);
            } 
          }
        }
      }
    }
  }
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  f = fopen("AlignmentCatalog.xml", "w");
  genxStartDocFile(w, f);
  genxStartElement(dddb);
    genxAddText(w, (gTYPE)"\n");
    genxStartElement(catalog);
      genxAddAttribute(name, (gTYPE)detector.c_str());
      genxAddText(w, (gTYPE)"\n  ");

      // Condition for whole detector
      genxStartElement(conditionref);
        std::string ref = "Alignment/Detectors.xml#"+ detector + "System";
        genxAddAttribute(href,(gTYPE) ref.c_str()); 
      genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");

      // Loop over stations
      for ( int i = 0; i != 2; ++i ) {
        genxStartElement(conditionref);
          std::string ref = "Alignment/Detectors.xml#"+ station[i];
          genxAddAttribute(href,(gTYPE) ref.c_str()); 
        genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
      }
      
      // Loop over stations
      for ( int i = 0; i != 2; ++i ) {
        // Loop over layers
        for ( int j = 0; j != 2; ++j ) {
          genxStartElement(conditionref);
            std::string ref = "Alignment/Detectors.xml#"+ station[i]
              +layer[2*i+j]+"Layer";
            genxAddAttribute(href,(gTYPE) ref.c_str()); 
          genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
        }
      }
      
      // Loop over stations
      for ( int i = 0; i != 2; ++i ) {
        // Loop over layers
        for ( int j = 0; j != 2; ++j ) {
          // Loop over regions
          for ( int k = 0; k != 3; ++k ) {
            int nModules = (region[k]=="R2") ? 3 + (2*i) : 6 ;
            // Loop over modules
            for ( int l = 0; l != nModules; ++l ) {
              // Loop over top/bottom
              for ( int m = 0; m != 2; ++m ) {
                genxStartElement(conditionref);
                std::string ref = "Alignment/Modules.xml#"+ station[i]
                  +layer[2*i+j]+"Layer"+region[k]+module[l]+tb[m];
                genxAddAttribute(href,(gTYPE) ref.c_str()); 
                genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
                // loop over sensors
                for (int n = 0; n != 7; ++n ) {                 
                  genxStartElement(conditionref);
                  std::string ref = "Alignment/Sensors.xml#"+ station[i]
                    +layer[2*i+j]+"Layer"+region[k]+module[l]+tb[m]+sensor[n];
                  genxAddAttribute(href,(gTYPE) ref.c_str()); 
                  genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
                }
              }
            }
          }
        }
      }
    genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w); genxAddText(w, (gTYPE)"\n");


  std::cout << "Written: AlignmentCatalog.xml, Detectors.xml, Modules.xml and Sensors.xml" << std::endl;
  std::cout << std::endl;

  return 0;
}

extern void makeAlignmentCondition( const std::string& detectorName )
{
  genxElement paramVector;
  genxElement condition; 
  genxAttribute name;
  genxAttribute type; 
  genxAttribute classID;

  if (!(paramVector=genxDeclareElement(w, NULL, (gTYPE)"paramVector", &status)))
    oops(w);
  if (!( condition = genxDeclareElement(w, NULL, (gTYPE)"condition", 
                                           &status))) oops(w);
  if (!(name = genxDeclareAttribute(w, NULL,(gTYPE) "name", &status))) oops(w);
  if (!(classID = genxDeclareAttribute(w, NULL,(gTYPE) "classID", &status)))
    oops(w);
  if (!(type = genxDeclareAttribute(w, NULL,(gTYPE) "type", &status))) oops(w);

  genxStartElement(condition);
  genxAddAttribute(classID,(gTYPE) "6");
  genxAddAttribute(name, (gTYPE)detectorName.c_str());
  genxAddText(w, (gTYPE)"\n    ");
    genxStartElement(paramVector);
    genxAddAttribute(name,(gTYPE)"dPosXYZ");
    genxAddAttribute(type,(gTYPE)"double");
    genxAddText(w,(gTYPE)"0 0 0");
    genxEndElement(w);
    genxAddText(w, (gTYPE)"\n    ");
    genxStartElement(paramVector);
    genxAddAttribute(name,(gTYPE)"dRotXYZ");
    genxAddAttribute(type,(gTYPE)"double");
    genxAddText(w,(gTYPE)"0 0 0");
    genxEndElement(w);
    genxAddText(w, (gTYPE)"\n  ");
  genxEndElement(w);
  genxAddText(w, (gTYPE)"\n  ");
        
}
