
#include <boost/lexical_cast.hpp>
#include <boost/assign/std/vector.hpp>
#include "genx.h"
#include <string>
#include <iostream>

/*
 g++ -I/afs/cern.ch/sw/lcg/external/Boost/1.32.0/slc3_ia32_gcc323/include/boost-1_32 -I/afs/cern.ch/sw/Gaudi/releases/GAUDI/GAUDI_v18r9/GaudiKernel/v20r10/ -Igenx/ ttCond.cpp genx/libgenx.a -o ttCond
*/

using namespace boost::assign;
using namespace boost;

#include "STChannelID.h"
using namespace LHCb;

const unsigned int nHybrid = 6;


void oops(genxWriter w)
{
  fprintf(stderr, "oops ERROR %s\n", genxLastErrorMessage(w));
  exit(1);
}

extern void vectorToString( std::string& output , std::vector<STChannelID>& channels);
 
int main(){


  std::string hybridsPerBoard = boost::lexical_cast<std::string>(nHybrid);

  std::vector<int> sectorChannelsA;
  sectorChannelsA += 1 ,1 ,1 ,1 ,1 ,1 ,
                     1 ,1 ,1 ,1 ,1 ,1 ,
                     1 ,1 ,1 ,1 ,1 ,1 ,
                     1 ,1 ,1 ,1 ,1 ,1 ,
                     2 ,2 ,2 ,2 ,2 ,2 ,
                     2 ,2 ,2 ,2 ,2 ,2 ,
                     2 ,2 ,2 ,2 ,2 ,2 ,
                     3 ,3 ,3 ,3 ,3 ,3 ,
                     3 ,3 ,3 ,3 ,3 ,3 ,
                     3 ,3 ,3 ,3 ,3 ,3 ,
                     3 ,3 ,3 ,3 ,3 ,3 ;

  std::vector<int> waferChannelsA;  
  waferChannelsA += 1 ,5 ,9 ,13 ,17 ,21 ,
                   2 ,6 ,10 ,14 ,18 ,22 ,
                   3 ,7 ,11 ,15 ,19 ,23 ,
                   4 ,8 ,12 ,16 ,20 ,24 ,
                   1 ,2 ,3 ,4 ,5 ,6 ,
                   7 ,8 ,9 ,10 ,11 ,12 ,
                   13 ,14 ,15 ,16 ,17 ,18 ,
                   1 ,5 ,9 ,13 ,17 ,21 ,
                   2 ,6 ,10 ,14 ,18 ,22 ,
                   3 ,7 ,11 ,15 ,19 ,23 ,
                   4 ,8 ,12 ,16 ,20 ,24 ;

  std::vector<int> sectorChannelsB;
  sectorChannelsB += 1 ,1 ,1 ,1 ,1 ,1 ,
                     1 ,1 ,1 ,1 ,1 ,1 ,
                     1 ,1 ,1 ,1 ,1 ,1 ,
                     1 ,1 ,1 ,1 ,1 ,1 ,
                     2 ,2 ,0, 2 ,2 ,0 ,
                     2 ,2 ,2 ,2 ,2 ,2 ,
                     2 ,2 ,2 ,2 ,2 ,2 ,
                     2 ,2 ,2 ,2 ,2 ,2 ,
                     2 ,2 ,0, 2 ,2 ,0 ,
                     3 ,3 ,3 ,3, 3, 3 ,
                     3 ,3 ,3 ,3 ,3 ,3 ,
                     3, 3, 3, 3, 3, 3 ,
                     3 ,3 ,3 ,3 ,3, 3;


  std::vector<int> waferChannelsB;
  waferChannelsB += 1 ,5 ,9 ,13 ,17 ,21 ,
                    2 ,6 ,10 ,14 ,18 ,22 ,
                    3 ,7 ,11 ,15 ,19 ,23 ,
                    4 ,8 ,12 ,16 ,20 ,24 ,
                    1 ,2 ,0, 3 ,4 ,0 ,
                    5 ,6 ,7 ,8 ,9 ,10 ,
                    11 ,12 ,13 ,14 ,15 ,16 ,
                    17 ,18 ,19 ,20 ,21 ,22 ,
                    23 , 24, 0, 25, 26, 0,
                    1 ,5 ,9 ,13 ,17 , 21,
                    2 ,6 ,10 ,14 ,18 ,22 ,
                    3 ,7 ,11 ,15, 19, 23,
                    4 ,8 ,12 ,16 ,20, 24;
                           
  // make vector of for TTa x layer channelIDs...
  std::vector<STChannelID> channels;
  for (unsigned int i = 0; i < waferChannelsA.size() ; ++i){
    STChannelID aChan(0,1,1, sectorChannelsA[i], waferChannelsA[i], 0 );
    channels.push_back(aChan);
  }
  // make string
  std::string TTaXMap;
  vectorToString(TTaXMap, channels);


  // same for TTa u
  channels.clear();
  for (unsigned int i = 0; i < waferChannelsA.size() ; ++i){
    STChannelID aChan(0,1,2, sectorChannelsA[i], waferChannelsA[i], 0 );
    channels.push_back(aChan);
  }
  // make string
  std::string TTaUMap;
  vectorToString(TTaUMap, channels);

  // same for TTb v
  channels.clear();
  for (unsigned int i = 0; i < waferChannelsB.size() ; ++i){
    if ( sectorChannelsB[i] != 0 ) {
      STChannelID aChan(0,2,1, sectorChannelsB[i], waferChannelsB[i], 0 );
      channels.push_back(aChan);
    } else {
      STChannelID aChan(0,0,0, 0, 0, 0 );
      channels.push_back(aChan);
    }
  }
  // make string
  std::string TTbVMap;
  vectorToString(TTbVMap, channels);


  // same for TTb x
  channels.clear();
  for (unsigned int i = 0; i < waferChannelsB.size() ; ++i){
    if ( sectorChannelsB[i] != 0 ) {
      STChannelID aChan(0,2,2, sectorChannelsB[i], waferChannelsB[i], 0 );
      channels.push_back(aChan);
    } else {
      STChannelID aChan(0,0,0, 0, 0, 0 );
      channels.push_back(aChan);
    }
  }
  // make string
  std::string TTbXMap;
  vectorToString(TTbXMap, channels);

  std::vector<std::string> layers;
  layers += "TTaX", "TTaU", "TTbV", "TTbX";
  std::string layerList;
  for (std::vector<std::string>::iterator iterS = layers.begin(); 
       iterS != layers.end(); ++iterS){
    layerList += " "+(*iterS)+"Map";
  } // iterS

  std::vector<std::string> output;
  output += TTaXMap, TTaUMap , TTbVMap, TTbXMap;


  typedef const unsigned char* gTYPE; 

  genxWriter w = genxNew(NULL, NULL, NULL);

  genxStatus status;
  
  genxElement param;
  genxElement paramVector;
  genxElement dddb;
  genxElement condition;
  genxElement catalog;
 
  genxAttribute name;
  genxAttribute type; 
  genxAttribute comment;
  genxAttribute classID;

  if (!( paramVector = genxDeclareElement(w, NULL, (gTYPE)"paramVector", &status))) oops(w);
  if (!(param = genxDeclareElement(w, NULL,(gTYPE) "param", &status))) oops(w);

  if (!(dddb =genxDeclareElement(w, NULL, (gTYPE)"DDDB", &status))) oops(w);
  if (!( catalog = genxDeclareElement(w, NULL, (gTYPE)"catalog", &status))) oops(w);
  if (!( condition = genxDeclareElement(w, NULL, (gTYPE)"condition", &status))) oops(w);

  if (!(name = genxDeclareAttribute(w, NULL,(gTYPE) "name", &status))) oops(w);
  if (!(comment = genxDeclareAttribute(w, NULL,(gTYPE) "comment", &status))) oops(w);
  if (!(classID = genxDeclareAttribute(w, NULL,(gTYPE) "classID", &status))) oops(w);
  if (!(type = genxDeclareAttribute(w, NULL,(gTYPE) "type", &status))) oops(w);

  std::string detector = "TT";

  genxStartDocFile(w, stdout);
  genxStartElement(dddb);
    genxAddText(w, (gTYPE)"\n");
    genxStartElement(catalog);
      genxAddAttribute(name, (gTYPE)detector.c_str());
      genxAddText(w, (gTYPE)"\n");
      genxStartElement(condition);
      genxAddAttribute(name,(gTYPE)"ReadoutMap");
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(paramVector);
        genxAddAttribute(name,(gTYPE)"layers");
        genxAddAttribute(type,(gTYPE)"string");
        genxAddAttribute(comment,(gTYPE)"List of layers");
	genxAddText(w,(gTYPE)layerList.c_str());
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(param);
        genxAddAttribute(name,(gTYPE)"hybridsPerBoard");
        genxAddAttribute(type,(gTYPE)"int");  
        genxAddAttribute(comment,(gTYPE)"# hybrids per Tell1 board");
	genxAddText(w,(gTYPE)hybridsPerBoard.c_str());
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(param);
        genxAddAttribute(name,(gTYPE)"nRegionsInTTa");
        genxAddAttribute(type,(gTYPE)"int");  
        genxAddAttribute(comment,(gTYPE)"# of regions A");
	genxAddText(w,(gTYPE)"2");
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");

      genxStartElement(paramVector);
        genxAddAttribute(name,(gTYPE)"nBoardsPerLayer");
        genxAddAttribute(type,(gTYPE)"int");  
        genxAddAttribute(comment,(gTYPE)"# Tell1 boards per layer");
	genxAddText(w,(gTYPE)"11 11 13 13");
      genxEndElement(w);
      genxAddText(w, (gTYPE)"\n");


      for (unsigned int i = 0 ; i < layers.size(); ++i){
        genxStartElement(paramVector);
	std::string tmp2 =  layers[i] + "Map";
          genxAddAttribute(name,(gTYPE)tmp2.c_str());
          genxAddAttribute(type,(gTYPE)"int");  
	  std::string tmp = "Readout Mapping for " + layers[i];
          genxAddAttribute(comment,(gTYPE)tmp.c_str());
          genxAddText(w, (gTYPE)"\n");
          genxAddText(w,(gTYPE)output[i].c_str());
          genxAddText(w, (gTYPE)"\n");
        genxEndElement(w);
	genxAddText(w, (gTYPE)"\n");
      } //
 
      genxAddText(w, (gTYPE)"\n");
    genxEndElement(w);
    genxAddText(w, (gTYPE)"\n");
   genxEndElement(w);
  genxAddText(w, (gTYPE)"\n");
  genxEndElement(w);
  genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  std::cout << " " << std::endl;

  
 std::cout << std::endl;


  int vecLoc = 0;
  for (unsigned int it = 0; it <11; ++it  ){
    std::ostringstream mess;
    mess << it;
    for (unsigned int ih = 0 ; ih < 6 ; ++ih, ++vecLoc){
      mess << " & (" << sectorChannelsA[vecLoc] << "," 
                     << waferChannelsA[vecLoc]<< ") ";   
    }
    mess << "\\\\ \\hline" ;
    std::cout << mess.str() << std::endl;
  }

  std::cout << std::endl;
  vecLoc = 0;
  for (unsigned int it = 0; it <13; ++it  ){
    std::ostringstream mess;
    mess << it;
    for (unsigned int ih = 0 ; ih < 6 ; ++ih, ++vecLoc){
      mess << " & (" << sectorChannelsB[vecLoc] << "," 
                     << waferChannelsB[vecLoc]<< ") ";   
    }
    mess << "\\\\ \\hline" ;
    std::cout << mess.str() << std::endl;
  }


  return 0;

}


extern void vectorToString( std::string& output , std::vector<STChannelID>& channels){
  
  for (unsigned int i = 0; i < channels.size() ; ++i){
    if ( channels[i].channelID() != 0 ) 
      output += " " + boost::lexical_cast<std::string>(channels[i].channelID());
    else 
      output += " 0000000";
    if ((i+1) % nHybrid == 0) output += " \n";  
  }
  
}

